package com.shigongli.common;

import com.shigongli.config.ScreenShotConfig;
import lombok.extern.slf4j.Slf4j;
import org.checkerframework.checker.nullness.qual.Nullable;
import org.openqa.selenium.*;
import org.openqa.selenium.chrome.ChromeDriver;
import org.openqa.selenium.chrome.ChromeOptions;
//import org.openqa.selenium.devtools.DevTools;
//import org.openqa.selenium.devtools.network.Network;
import org.openqa.selenium.support.ui.ExpectedCondition;
import org.openqa.selenium.support.ui.WebDriverWait;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
//import java.util.Optional;

@Slf4j
@Component
public class ScreenShotUtil {
    @Autowired
    OSSUtils ossUtils;

    @Autowired
    ScreenShotConfig config;

    public String shot(String url) throws Exception {
        //驱动地址
        System.setProperty("webdriver.chrome.driver", config.getDriver());

        WebDriver driver = null;
        ChromeOptions options=new ChromeOptions();
        //设置 chrome 的无头模式
        options.addArguments("--headless");
        options.addArguments("--disable-gpu");
        options.addArguments("--no-sandbox");
        options.addArguments("--disable-dev-shm-usage");
        options.addArguments("--start-maximized");
        options.addArguments("--disable-application-cache");
        options.addArguments("--lang=zh");
        options.addArguments(String.format("--window-size=%s,%s", config.getWidth(), config.getHeight()));
        //启动一个 chrome 实例
        driver = new ChromeDriver(options);

        //访问网址
        log.info(String.format("访问网址: %s", url));
//        webDriver.get(url);

//        DevTools devTools = driver.getDevTools();
//        devTools.createSession();
//        devTools.send(Network.enable(Optional.empty(), Optional.empty(), Optional.of(100000000)));
//        devTools.send(Network.setCacheDisabled(true));
//        devTools.addListener(Network.responseReceived(), responseReceived -> Assert.assertEquals(false, responseReceived.getResponse().getFromDiskCache()));

        driver.get(url);

//        devTools.send(Network.clearBrowserCache());

        // 设置超时等待
        try {
            WebDriverWait webDriverWait = new WebDriverWait(driver, config.getWait() / 1000, 200);
            webDriverWait.until(new ExpectedCondition<WebElement>() {
                public @Nullable WebElement apply(@Nullable WebDriver driver) {
                    WebElement el = null;
                    try {
                        WebDriver wf = driver.switchTo().frame(0);
                        log.info("Got Web Iframe {}", wf);
                        el = wf.findElement(By.cssSelector("body"));
                        log.info("Got Web Element {}", el);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                    return el;
                }
            });
        } catch (Exception e) {
            driver.quit();
            throw e;
        }

//        webDriver.manage().timeouts().implicitlyWait(config.getWait(), TimeUnit.MILLISECONDS);

        //截取全屏
        File f  = ((TakesScreenshot)driver).getScreenshotAs(OutputType.FILE);

        // 存入 oss
        String imageUrl = ossUtils.putObject(f);
        log.info("Got image {}", imageUrl);

        //退出
        driver.quit();

        return imageUrl;
    }
}
